@@ -88,7 +88,8 @@ from sklearn.utils.validation import check_X_y, check_array, check_is_fitted
 
 from imodels.rule_set.rule_set import RuleSet
 from imodels.util.convert import tree_to_rules
-from imodels.util.rule import replace_feature_name, enum_features
+from imodels.util.rule import replace_feature_name, get_feature_dict, Rule
+from imodels.util.extract import extract_skope
 from imodels.util.score import score_oob
 from imodels.util.prune import prune_mins, deduplicate
 
@@ -245,7 +246,7 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
         self.random_state = random_state
         self.verbose = verbose
 
-    def fit(self, X, y, feature_names=None, sample_weight=None) -> 'SkopeRulesClassifier':
+    def fit(self, X, y, feature_names=None, sample_weight=None):
         """Fit the model according to the given training data.
 
         Parameters
@@ -317,20 +318,18 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
                 raise ValueError("max_samples must be in (0, 1], got %r" % self.max_samples)
             max_samples = int(self.max_samples * X.shape[0])
         self.max_samples_ = max_samples
-        self._max_depths = self.max_depth if isinstance(self.max_depth, Iterable) else [self.max_depth]
 
-        self.feature_names_, self.feature_dict_ = enum_features(X, feature_names)
+        self.feature_dict_ = get_feature_dict(X.shape[1], feature_names)
+        self.feature_placeholders = list(self.feature_dict_.keys())
+        self.feature_names = list(self.feature_dict_.values())
 
-        self.tree_generators = self._get_tree_ensemble()
-        self._fit_tree_ensemble(X, y)
-
-        extracted_rules = self._extract_rules()
+        extracted_rules, self.estimators_samples_, self.estimators_features_ = self._extract_rules(X, y)
         scored_rules = self._score_rules(X, y, extracted_rules)
         self.rules_ = self._prune_rules(scored_rules)
 
         self.rules_without_feature_names_ = self.rules_
         self.rules_ = [
-            (replace_feature_name(rule, self.feature_dict_), perf) for rule, perf in self.rules_
+            replace_feature_name(rule, self.feature_dict_) for rule in self.rules_
         ]
         return self
 
@@ -350,13 +349,16 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
             be considered as an outlier according to the selected rules.
         """
 
-        return np.array((self.eval_weighted_rule_sum(X) > 0), dtype=int)
+        return np.argmax(self.predict_proba(X), axis=1)
 
     def predict_proba(self, X) -> np.ndarray:
         '''Predict probability of a particular sample being an outlier or not
 
         '''
-        y = self.rules_vote(X) / len(self.rules_without_feature_names_)
+        weight_sum = np.sum([w[0] for (r, w) in self.rules_without_feature_names_])
+        if weight_sum == 0:
+            return np.vstack((np.ones(X.shape[0]), np.zeros(X.shape[0]))).transpose()
+        y = self.eval_weighted_rule_sum(X) / weight_sum
         return np.vstack((1 - y, y)).transpose()
 
     def rules_vote(self, X) -> np.ndarray:
@@ -379,8 +381,7 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
 
         """
         # Check if fit had been called
-        check_is_fitted(self, ['rules_', 'estimators_', 'estimators_samples_',
-                               'max_samples_'])
+        check_is_fitted(self, ['rules_', 'estimators_samples_', 'max_samples_'])
 
         # Input validation
         X = check_array(X)
@@ -391,8 +392,8 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
                              " Please reshape your data."
                              % (X.shape[1], self.n_features_))
 
-        df = pandas.DataFrame(X, columns=self.feature_names_)
-        selected_rules = self.rules_
+        df = pandas.DataFrame(X, columns=self.feature_placeholders)
+        selected_rules = self.rules_without_feature_names_
 
         scores = np.zeros(X.shape[0])
         for (r, _) in selected_rules:
@@ -421,8 +422,7 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
 
         """
         # Check if fit had been called
-        check_is_fitted(self, ['rules_', 'estimators_', 'estimators_samples_',
-                               'max_samples_'])
+        check_is_fitted(self, ['rules_', 'estimators_samples_', 'max_samples_'])
 
         # Input validation
         X = check_array(X)
@@ -433,14 +433,14 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
                              " Please reshape your data."
                              % (X.shape[1], self.n_features_))
 
-        df = pandas.DataFrame(X, columns=self.feature_names_)
+        df = pandas.DataFrame(X, columns=self.feature_placeholders)
         selected_rules = self.rules_without_feature_names_
 
         scores = np.zeros(X.shape[0])
         for (k, r) in enumerate(list((selected_rules))):
-            scores[list(df.query(r[0]).index)] = np.maximum(
+            scores[list(df.query(r.rule).index)] = np.maximum(
                 len(selected_rules) - k,
-                scores[list(df.query(r[0]).index)])
+                scores[list(df.query(r.rule).index)])
 
         return scores
 
@@ -469,71 +469,27 @@ class SkopeRulesClassifier(BaseEstimator, RuleSet):
         return np.array((self.score_top_rules(X) > len(self.rules_) - n_rules),
                         dtype=int)
 
-    def _get_tree_ensemble(self) -> Union[List[BaggingClassifier], List[BaggingRegressor]]:
+    def _extract_rules(self, X, y) -> Tuple[List[str], List[np.array], List[np.array]]:
+        return extract_skope(X, y,
+                             feature_names=self.feature_placeholders,
+                             sample_weight=self.sample_weight,
+                             n_estimators=self.n_estimators,
+                             max_samples=self.max_samples_,
+                             max_samples_features=self.max_samples_features,
+                             bootstrap=self.bootstrap,
+                             bootstrap_features=self.bootstrap_features,
+                             max_depths=self.max_depth,
+                             max_depth_duplication=self.max_depth_duplication,
+                             max_features=self.max_features,
+                             min_samples_split=self.min_samples_split,
+                             n_jobs=self.n_jobs,
+                             random_state=self.random_state,
+                             verbose=self.verbose)
 
-        for ensemble_class, tree_class in [
-            (BaggingClassifier, DecisionTreeClassifier), (BaggingRegressor, DecisionTreeRegressor)
-        ]:
+    def _score_rules(self, X, y, rules) -> List[Rule]:
+        return score_oob(X, y, rules, self.estimators_samples_, self.estimators_features_, self.feature_placeholders)
 
-            ensembles = []
-
-            for max_depth in self._max_depths:
-                bagging_clf = ensemble_class(
-                    base_estimator=tree_class(
-                        max_depth=max_depth,
-                        max_features=self.max_features,
-                        min_samples_split=self.min_samples_split
-                    ),
-                    n_estimators=self.n_estimators,
-                    max_samples=self.max_samples_,
-                    max_features=self.max_samples_features,
-                    bootstrap=self.bootstrap,
-                    bootstrap_features=self.bootstrap_features,
-                    # oob_score=... XXX may be added
-                    # if selection on tree perf needed.
-                    # warm_start=... XXX may be added to increase computation perf.
-                    n_jobs=self.n_jobs,
-                    random_state=self.random_state,
-                    verbose=self.verbose
-                )
-                ensembles.append(bagging_clf)
-
-        return ensembles
-
-    def _fit_tree_ensemble(self, X, y) -> None:
-        y_reg = y
-        if self.sample_weight is not None:
-            sample_weight = check_array(self.sample_weight, ensure_2d=False)
-            weights = sample_weight - sample_weight.min()
-            contamination = float(sum(y)) / len(y)
-            y_reg = (
-                    pow(weights, 0.5) * 0.5 / contamination * (y > 0) -
-                    pow((weights).mean(), 0.5) * (y == 0)
-            )
-            y_reg = 1. / (1 + np.exp(-y_reg))  # sigmoid
-
-        for e in self.tree_generators[:len(self.tree_generators) // 2]:
-            e.fit(X, y)
-
-        for e in self.tree_generators[len(self.tree_generators) // 2:]:
-            e.fit(X, y_reg)
-
-    def _extract_rules(self):
-        self.estimators_, self.estimators_samples_, self.estimators_features_ = [], [], []
-        for ensemble in self.tree_generators:
-            self.estimators_ += ensemble.estimators_
-            self.estimators_samples_ += ensemble.estimators_samples_
-            self.estimators_features_ += ensemble.estimators_features_
-
-        extracted_rules = []
-        for estimator, features in zip(self.estimators_, self.estimators_features_):
-            extracted_rules.append(tree_to_rules(estimator, np.array(self.feature_names_)[features]))
-        return extracted_rules
-
-    def _score_rules(self, X, y, rules):
-        return score_oob(X, y, rules, self.estimators_samples_, self.estimators_features_, self.feature_names_)
-
-    def _prune_rules(self, rules):
+    def _prune_rules(self, rules) -> List[Rule]:
         return deduplicate(
             prune_mins(rules, self.precision_min, self.recall_min),
             self.max_depth_duplication
