@@ -4,7 +4,7 @@ from warnings import warn
 import pandas as pd
 import numpy as np
 from sklearn.utils import indices_to_mask
-from sklearn.linear_model import Lasso
+from sklearn.linear_model import Lasso, LogisticRegression
 from sklearn.linear_model._coordinate_descent import _alpha_grid
 from sklearn.model_selection import KFold
 
@@ -46,13 +46,14 @@ def score_oob(X,
 
         # Add OOB performances to rules:
         scored_rules += [
-            Rule(r, args=_eval_rule_perf(r, X_oob, y_oob)) for r in set(curr_rules)
+            Rule(r, args=_eval_rule_perf(r, X_oob, y_oob))
+            for r in set(curr_rules)
         ]
 
     return scored_rules
 
 
-def _eval_rule_perf(rule, X, y) -> Tuple[float, float]:
+def _eval_rule_perf(rule: str, X, y) -> Tuple[float, float]:
     detected_index = list(X.query(rule).index)
     if len(detected_index) <= 1:
         return (0, 0)
@@ -64,16 +65,28 @@ def _eval_rule_perf(rule, X, y) -> Tuple[float, float]:
     return y_detected.mean(), float(true_pos) / pos
 
 
-def score_lasso(X, y, rules: List[str], alphas=None, cv=3, max_rules=2000, random_state=None) -> Tuple[List[Rule], Lasso]:
+def score_lasso(X, y, rules: List[str], alphas=None, cv=3,
+                prediction_task='regression',
+                max_rules=2000, random_state=None) -> Tuple[List[Rule], List[float], float]:
     if alphas is None:
-        alphas = _alpha_grid(X, y) 
+        if prediction_task == 'regression':
+            alphas = _alpha_grid(X, y)
+        elif prediction_task == 'classification':
+            alphas = [1 / alpha
+                     for alpha in np.logspace(-4, 4, num=10, base=10)]
 
     coef_zero_threshold = 1e-6 / np.mean(np.abs(y))
     mse_cv_scores = []
     nonzero_rule_coefs_count = []
     kf = KFold(cv)
-    for alpha in alphas: # alphas are sorted from largest to smallest
-        m = Lasso(alpha=alpha, random_state=random_state)
+    
+    # alphas are sorted from most reg. to least reg.
+    for alpha in alphas: 
+        
+        if prediction_task == 'regression':
+            m = Lasso(alpha=alpha, random_state=random_state)
+        else:
+            m = LogisticRegression(penalty='l1', C=1/alpha, solver='liblinear')
         mse_cv = 0
         for train_index, test_index in kf.split(X):
             X_train, X_test = X[train_index], X[test_index]
@@ -83,21 +96,26 @@ def score_lasso(X, y, rules: List[str], alphas=None, cv=3, max_rules=2000, rando
         
         m.fit(X, y)
         
-        rule_count = sum(np.abs(m.coef_) > coef_zero_threshold)
+        rule_count = np.sum(np.abs(m.coef_.flatten()) > coef_zero_threshold)
         if rule_count > max_rules:
             break
         nonzero_rule_coefs_count.append(rule_count)
         mse_cv_scores.append(mse_cv / cv)
     
     best_alpha = alphas[np.argmin(mse_cv_scores)]
-    lscv = Lasso(alpha=best_alpha, random_state=random_state, max_iter=2000)
+    if prediction_task == 'regression':
+        lscv = Lasso(alpha=best_alpha, random_state=random_state, max_iter=2000)
+    else:
+        lscv = LogisticRegression(penalty='l1', C=1/best_alpha, solver='liblinear',
+                                  random_state=random_state, max_iter=200)
     lscv.fit(X, y)
 
-    coefs = list(lscv.coef_[:-len(rules)])
+    coef_ = lscv.coef_.flatten()
+    coefs = list(coef_[:-len(rules)])
     support = np.sum(X[:, -len(rules):], axis=0) / X.shape[0]
 
     nonzero_rules = []
-    for r, w, s in zip(rules, lscv.coef_[-len(rules):], support):
+    for r, w, s in zip(rules, coef_[-len(rules):], support):
         if abs(w) > coef_zero_threshold:
             nonzero_rules.append(Rule(r, args=[w], support=s))
             coefs.append(w)
